#ifndef __GST_URIDOWNLOADER_DEBUG_H__
#define __GST_URIDOWNLOADER_DEBUG_H__

#include <gst/gst.h>

G_BEGIN_DECLS

GST_DEBUG_CATEGORY_EXTERN (uridownloader_debug);

G_END_DECLS

#endif /* __GST_URIDOWNLOADER_DEBUG_H__ */
